# Edge minimization in de Bruijn graphs
This repository contains an implementation and a benchmark for de Bruijn graph,
edge minimization. The algorithms are described in

	De Bruijn graph edge minimization

	by Uwe Baier, Thomas Büchler, Enno Ohlebusch and Pascal Weber
	(hopefully to appear in DCC 2020)

## What is contained
This bundle of files consist of the following parts:
1. The algorithms required to minimize de Bruijn graphs and construct
   a tunneled FM index from it, contained in the `include`- and `lib`-directory
2. Data for reproducible computation results, see `rcrdata`-directory
3. Utility programs to construct and test the tunneled FM index
4. A benchmark script to test the construction algorithms

## Requirements
To compile the program(s), you need a modern c++11 ready compiler such as 
[gcc](https://gcc.gnu.org/) version 4.7 or newer.

## Prerequisites
The software uses the Succinct Data Structure Library [SDSL-LITE](https://github.com/simongog/sdsl-lite) by Simon Gog.
You have to specify the SDSL-LITE path with the "lib" and "include" folder by

```
export SDSLLITE="path/to/sdsllite/"
```

## Installation
Just call the command `make`. It should produce four executables:
- `dbg_edgecount.x`: program to measure the amount of edges of a DBG given an order k and an input file
- `dbg_edgespectrum.x`: program similar to dbg_edgecount, but prints for the range 1 to k the edges per DBG
- `tfm_index_construct.x`: program for constructing a tunneled FM index
- `tfm_index_invert.x`: program for inverting a tunneled FM index, that is, reconstruct the original string



##Benchmark
A benchmark is contained. To run it, configure `benchmark.config` as required and
run the benchmark by calling `make benchmark`. All timings are given in milliseconds,
all memory peaks in bytes.
If you want to store the results, try `make -s benchmark > result.dat` to store results in
`result.dat`.

##Reproducible computation results
To reproduce the computation results from the paper, call `make rcrbenchmark` which
should create two .dat files named `rcrbenchmark.dat` containing benchmark results as
well as `rcredgespectrum.dat` which for each file contains the edge spectrum.
